%error = forceRequiredAddon("Item_Sports");

if (%error == $Error::Addon_NotFound) {
	error("ERROR: Server_Chaos_Dunk - Required add-on Item_Sports not found!");
	return;
}

exec("./support.cs");

$dunkInProgress = 0;

datablock AudioProfile(dunk_thunderSound1) {
	filename = "Add-Ons/Server_Chaos_Dunk/sound/thunder1.wav";
	description = AudioClose3d;
	is3D = false;
	preload = true;
};

datablock AudioProfile(dunk_thunderSound2) {
	filename = "Add-Ons/Server_Chaos_Dunk/sound/thunder2.wav";
	description = AudioClose3d;
	is3D = false;
	preload = true;
};

datablock AudioProfile(dunk_sirenSound) {
	filename = "Add-Ons/Server_Chaos_Dunk/sound/siren.wav";
	description = AudioClose3d;
	is3D = false;
	preload = true;
};

datablock AudioProfile(dunk_explosionSound) {
	filename = "Add-Ons/Server_Chaos_Dunk/sound/explosion.wav";
	description = AudioClose3d;
	is3D = false;
	preload = true;
};

datablock AudioProfile(dunk_beginSound) {
	filename = "Add-Ons/Server_Chaos_Dunk/sound/begin.wav";
	description = AudioClose3d;
	is3D = false;
	preload = true;
};

function dunkSoundLoop(%count) {
	cancel($dunkSoundLoop);

	if ($dunkInProgress < 1) {
		%count = 0;
		return;
	}

	if ((%count % 10) == 0) {
		%count = 0;
		serverPlay2D(dunk_sirenSound);
	}
	
	if (getRandom(1, 3) == 1) {
		serverPlay2D(dunk_thunderSound @ getRandom(1, 2));
		for (%i = 0; %i < clientGroup.getCount(); %i++) {
			%client = clientGroup.getObject(%i);
			if (isObject(%player = %client.player)) {
				%player.setWhiteOut(getRandom() / 2);
			}
		}
	}
	
	%count++;
	
	$dunkSoundLoop = schedule(1000, 0, dunkSoundLoop, %count);
}

function player::cd_Explode(%player) {
	if (isObject(%player) && %player.isDunking) {
		cancel(%player.cdTick);
		
		%player.dunkReady = true;
		
		%player.setHealth(%player.getDatablock().maxDamage);
		%player.burnPlayer(60);
		%player.addVelocity("0 0 -2500");
	}
}

function player::cd_Tick(%player) {
	cancel(%player.cdTick);
	
	if (!isObject(%player))
		return;
	
	if (%player.getMountedImage(0).item !$= "basketballItem") {
		cancel(%player.cdTick);
		cancel(%player.cdExplodeTick);
		%player.isDunking = false;
		$dunkInProgress--;
		return;
	}
	
	%trans = %player.getTransform();
	
	%pos = getWords(%trans, 0, 2);
	%pos = vectorAdd(%pos, "0 0 0.1");
	
	%ang = axisToEuler(getWords(%trans, 3, 6));
	%ang = vectorAdd(%ang, "0 0 5");
	%ang = eulerToAxis(%ang);
	
	%player.setTransform(%pos SPC %ang);
	%player.setVelocity("0 0 0");
	
	%player.cdTick = %player.schedule(10, "cd_Tick");
	%player.cdExplodeTick = %player.schedule(30000, "cd_Explode");
}

function player::chaosDunk(%player) {
	if (!isObject(%player))
		return;
	
	%client = %player.client;
	
	%player.isDunking = true;
	$dunkInProgress++;
	
	serverPlay3D(dunk_beginSound, %player.getHackPosition());
	
	%client.camera.setMode("corpse", %player);
	%client.camera.setControlObject(%client.camera);
	%client.setControlObject(%client.camera);
	
	%player.cd_Tick();
	dunkSoundLoop(5);
}

function player::abortChaosDunk(%player) {
	if (%player.isDunking) {
		cancel(%player.cdTick);
		cancel(%player.cdExplodeTick);
		%player.isDunking = false;
		$dunkInProgress--;
	}
}

if (isPackage(chaos_Dunk))
	deactivatePackage(chaos_Dunk);

package chaos_Dunk {
	
	function observer::onTrigger(%observer, %obj, %trigger, %state) {
		%client = %obj.getControllingClient();
		%player = %client.player;
		
		if (isObject(%player) && %player.isDunking)
			return;

		parent::onTrigger(%observer, %obj, %trigger, %state);
	}
	
	function serverCmdLight(%client) {
		%player = %client.player;
		
		if (isObject(%player) && !%player.isDunking) {
			if (%player.getMountedImage(0).item $= "basketballItem") {
				if (%client.minigame || %client.isAdmin) {
					%player.chaosDunk();
					return;
				}
			}
		}
		
		parent::serverCmdLight(%client);
	}
	
	function Armor::onImpact(%armor, %player, %col, %vel, %speed) {
		parent::onImpact(%armor, %player, %col, %vel, %speed);
		
		if (isObject(%player)) {
			if (%player.dunkReady) {
				%player.kill();
				return;
			}
		}
	}
	
	function gameConnection::onDeath(%client, %kObj, %killer, %type, %loc) {
		%player = %client.player;
		
		if (%player.dunkReady) {
			if (%client.minigame) {
				for (%i = 0; %i < %client.minigame.numMembers; %i++) {
					%vic = %client.minigame.member[%i];
					if (isObject(%vicObj = %vic.player)) {
						if (minigameCanDamage(%player, %vicObj)) {
							%vicObj.setWhiteOut(4);
							%vicObj.kill();
						}
					}
				}
			} else if (%client.isAdmin) {
				for (%i = 0; %i < clientGroup.getCount(); %i++) {
					%vic = clientGroup.getObject(%i);
					if (isObject(%vicObj = %vic.player))
						%vicObj.setWhiteOut(2);
				}
			}
			
			serverPlay2D(dunk_explosionSound);
			$dunkInProgress--;
		} else if (%player.isDunking) {
			%player.abortChaosDunk();
		}
		
		parent::onDeath(%client, %kObj, %killer, %type, %loc);
	}
	
	function gameConnection::onClientLeaveGame(%client) {
		%player = %client.player;
		
		if (isObject(%player) && %player.isDunking)
			%player.abortChaosDunk();
		
		parent::onClientLeaveGame(%client);
	}
	
};

activatePackage(chaos_Dunk);